from __future__ import unicode_literals

import mysql.connector
from mysql.connector import errorcode
import time

class DBConnector():
    """
    Connects to database, sends and receives messages
    """

    def __init__(self, dbUser, dbPwd, dbHost, dbName, trigger, bot_id):
        """
        Initializes the DBConnector, establishes the database connection, sets instance variables

        :param dbUser: database username
        :type name: str.
        :param dbPwd: database userpassword
        :type state: str.
        :param dbHost: database host address
        :type state: str.
        :param dbName: database name
        :type state: str.
        :param trigger: string to trigger the bot
        :type state: str.
        :param bot_id: humhub user id of the bot user
        :type state: int.
        """
        self.cnx = self.connectToDB(dbUser, dbPwd, dbHost, dbName)
        self.current_id = self.getCurrentID()
        self.trigger = trigger
        self.bot_id = bot_id

    def connectToDB(self, dbUser, dbPwd, dbHost, dbName):
        """
        Establishes connection to the database

        :param dbUser: database username
        :type name: str.
        :param dbPwd: database userpassword
        :type state: str.
        :param dbHost: database host address
        :type state: str.
        :param dbName: database name
        :type state: str.
        :returns: MySQLConnection -- Instance of class MySQLConnection
        """
        try:
            cnx = mysql.connector.connect(user=dbUser, password=dbPwd, host=dbHost, database=dbName, autocommit=True)
        except mysql.connector.Error as err:
            if err.errno == errorcode.ER_ACCESS_DENIED_ERROR:
                print("Something is wrong with your user name or password")
            elif err.errno == errorcode.ER_BAD_DB_ERROR:
                print("Database does not exist")
            else:
                print(err)
        else:
            return cnx

    def getCurrentID(self):
        """
        Gets the current max message ID from Humhub

        :returns: int -- Current max message ID
        """
        cursor = self.cnx.cursor()
        query = "SELECT MAX(id) FROM message_entry"
        cursor.execute(query)
        return cursor.fetchone()[0]

    def checkNewDBMessages(self):
        """
        Checks for new messages, saves the new message ID if a new message is available

        :returns: bool -- True when new bot message is available, False if there is no new message
        """
        cursor = self.cnx.cursor()
        query = ("SELECT MAX(id) FROM message_entry WHERE user_id <> %(bot_id)s AND (content LIKE %(trigger)s OR message_entry.message_id IN "
            "(SELECT DISTINCT message_entry.message_id FROM message_entry JOIN user_message "
            "ON message_entry.message_id=user_message.message_id WHERE user_message.user_id = 5 ORDER BY message_entry.message_id))")
        data = {
            'bot_id': self.bot_id,
            'trigger': self.trigger + '%', # wildcard for SQL
        }
        cursor.execute(query, data)
        new_id = cursor.fetchone()[0]
        cursor.close()
        if self.current_id < new_id:
            self.current_id = new_id
            return True
        else:
            return False

    def getNewDBMessage(self):
        """
        Gets the newest message

        :returns: dictionary -- Containing the message itself as string and the conversation ID
        """
        cursor = self.cnx.cursor()
        query = "SELECT message_id, content FROM message_entry WHERE (user_id <> 5 AND id = {})".format(self.current_id)
        cursor.execute(query)
        result = cursor.fetchone()
        message_id = result[0]
        if result[1][:len(self.trigger)] == self.trigger:
            message = result[1][len(self.trigger):].strip()
        else:
            message = result[1].strip()
        cursor.close()
        messagedata = {
            'message': message,
            'message_id': message_id
        }
        return messagedata

    def saveToDB(self, messagedata):
        """
        Saves reply message from Rasa_Core to db
        
        :param messagedata: Containing the reply from Rasa as string and the conversation id
        :type state: dictionary.
        """
        cursor = self.cnx.cursor()
        query = ("INSERT INTO message_entry(message_id, user_id, content, created_at, created_by, updated_at, updated_by) "
            "VALUES (%(msg_id)s, %(bot_id)s, %(message)s, NOW(), %(bot_id)s, NOW(), %(bot_id)s)")
        data = {
          'msg_id': messagedata['message_id'],
          'bot_id': self.bot_id,
          'message': messagedata['reply'],
        }
        try:
            cursor.execute(query, data)
            self.cnx.commit()
            cursor.close()
        except mysql.connector.Error as err:
            if err.errno == errorcode.ER_ACCESS_DENIED_ERROR:
                print("Something is wrong with your user name or password")
            elif err.errno == errorcode.ER_BAD_DB_ERROR:
                print("Database does not exist")
            else:
                print(err)



from __future__ import unicode_literals
from rasahub.handler.dbconnector import DBConnector
from rasahub.handler.rasaconnector import RasaConnector

import argparse

def create_argument_parser():
    parser = argparse.ArgumentParser(
            description='starts rasahub')
    parser.add_argument(
            '-dbu', '--dbuser',
            required=True,
            type=str,
            help="database username")
    parser.add_argument(
            '-dbp', '--dbpwd',
            required=True,
            type=str,
            help="database user password")
    parser.add_argument(
            '-dbh', '--dbhost',
            type=str,
            default='127.0.0.1',
            help="database hostname")
    parser.add_argument(
            '-dbn', '--dbname',
            required=True,
            type=str,
            help="database name")
    parser.add_argument(
            '-t', '--trigger',
            type=str,
            default='!bot',
            help="bot trigger string")
    parser.add_argument(
            '-id', '--botid',
            required=True,
            type=str,
            help="bots user id in humhub")
    parser.add_argument(
            '-rh', '--rasahost',
            type=str,
            default='127.0.0.1',
            help="host address of rasa_core")
    parser.add_argument(
            '-rp', '--rasaport',
            type=int,
            default=5020,
            help="port of rasa_core rasahubchannel")
    return parser

def main():
    """
    Initializes DBConnector and RasaConnector, handles messages
    """
    arg_parser = create_argument_parser()
    cmdline_args = arg_parser.parse_args()

    dbconn = DBConnector(cmdline_args.dbuser,
                         cmdline_args.dbpwd,
                         cmdline_args.dbhost,
                         cmdline_args.dbname,
                         cmdline_args.trigger,
                         cmdline_args.botid)
    print("Connected to database. Waiting for socket connection from Rasa on port {}".format(cmdline_args.rasaport))

    rasaconn = RasaConnector(cmdline_args.rasahost,
                             cmdline_args.rasaport)

    while (True):
        if (dbconn.checkNewDBMessages()):
            inputmsg = dbconn.getNewDBMessage()
            print("Input from db: {}".format(inputmsg))
            reply = rasaconn.getReply(inputmsg)
            print("Reply from rasa: {}".format(reply))
            dbconn.saveToDB(reply)



from __future__ import unicode_literals

import json
import socket
import time

class RasaConnector():
    """
    Connects to Rasa, send and receive messages
    """

    def __init__(self, rasaIP, rasaPort):
        """
        Initializes the RasaConnector, establishes the server socket

        :param rasaIP: IP address of Rasa_Core instance
        :type name: str.
        :param rasaPort: Port number set in Rasa_Core RasahubInputChannel
        :type state: int.
        """
        socket = self.establishSocket(rasaIP, rasaPort)
        socket.listen(5)
        c, addr = socket.accept()
        print("Connection established: %s" % str(addr))
        self.con = c

    def establishSocket(self, ip, port):
        """
        Establishes the socket

        :param ip: IP address of Rasa_Core instance
        :type name: str.
        :param port: Port number set in Rasa_Core RasahubInputChannel
        :type state: int.
        :returns: socket -- the serversocket
        """
        newSocket = socket.socket()
        newSocket.bind((ip, port))
        return newSocket

    def getReply(self, messagedata):
        """
        Sends message to Rasa_Core, returns reply

        :param messagedata: Input message as string and conversation ID
        :type name: dictionary.
        :returns: dictionary - the reply from Rasa as string and conversation ID as string
        """
        message = messagedata['message']
        self.con.send(json.dumps(messagedata).encode())
        replied = 0
        timeout = time.time() + 5
        while True:
            if time.time() > timeout:
                return ""
            else:
                reply = self.con.recv(1024).decode('utf-8')
                if reply:
                    replydata = {
                        'reply': reply,
                        'message_id': messagedata['message_id']
                    }
                    return replydata



from setuptools import setup, find_packages

setup(
    name='rasahub',
    version='0.0.12',
    description='Rasahub connects Rasa_core to Humhub Mail',
    author='Christian Frommert',
    author_email='christian.frommert@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development',
    ],
    keywords='humhub rasa rasa_core',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['rasa-core', 'rasa_nlu', 'mysql-connector==2.1.4'],
    entry_points={
       'console_scripts': [
           'rasahub = rasahub:main',
       ],
    }
)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import socket
import json

from rasa_core.channels.channel import UserMessage
from rasa_core.channels.channel import InputChannel, OutputChannel

class RasahubInputChannel(InputChannel):
    """
    Receives Messages via socket
    """
    def __init__(self, ip, port):
        self.socket = socket.socket()
        self.socket.connect((ip, port))

    def start_async_listening(self, message_queue):
        self._record_messages(message_queue.enqueue)

    def start_sync_listening(self, message_handler):
        self._record_messages(message_handler)

    def _record_messages(self, on_message):
        while True:
            try:
                text = json.loads(self.socket.recv(1024).decode('utf-8')) # gets message data: message and message_id
                on_message(UserMessage(text['message'], RasahubOutputChannel(self.socket), text['message_id']))
            except socket.timeout:
                continue

class RasahubOutputChannel(OutputChannel):
    """
    Sends messages via socket
    """
    def __init__(self, socket):
        self.socket = socket
    def send_text_message(self, recipient_id, message):
        self.socket.sendall(message.encode('utf-8'))
